
wilsonLU = function (c, y, n, alpha) -> (L, U)

    g = c.length;

    p = y ./ n;

    l = zeros(g);
    u = zeros(g);

    cv = normal.quantile(1 - alpha / 2);

    for k = 1 : g
         t = cv*sqrt((p[k] * (1-p[k]) + cv^2/(4*n[k])) / n[k]);
         l[k] = (p[k] + cv^2/(2*n[k]) - t) / (1 + cv^2/n[k]);
         u[k] = (p[k] + cv^2/(2*n[k]) + t) / (1 + cv^2/n[k]);
    end

    S = 0;
    T = 0;

    for k = 1 : g
         S += (c[k] * p[k] - min(c[k] * l[k], c[k] * u[k])) ^ 2;
         T += (c[k] * p[k] - max(c[k] * l[k], c[k] * u[k])) ^ 2;
    end

    P = dot(c, p);

    L = P - sqrt(S);
    U = P + sqrt(T);
end



waldAdjLU = function (c, y, n, alpha) -> (L, U)

    k = length(find(c));

    p = y ./ n;

    phat = (y + 2/k) ./ (n + 4/k);

    cv = normal.quantile(1 - alpha / 2);

    h = cv * sqrt(sum(c.^2 .* phat .* (1-phat) ./ (n + 4/k)));

    P = dot(c, phat);

    L = P - h;
    U = P + h;


end


waldLU = function (c, y, n, alpha) -> (L, U)
    p = y ./ n;

    cv = normal.quantile(1 - alpha / 2);

    h = cv * sqrt(sum(c.^2 .* p .* (1-p) ./ n));

    P = dot(c, p);

    L = P - h;
    U = P + h;
end

fo = fopen("ci.txt", "w");

y = [20,14,27,19];
n = [30,30,30,30];
alpha = 0.05;

fo.printf("\n Example 1\n");
fo.printf("y = [20,14,27,19], n=[30,30,30,30], alpha=0.05\n");

c = [1,-1,-1,1];

fo.writeline("c = [1,-1,-1,1]");

fo.printf("\n");
ci = waldLU(c, y, n, alpha);
fo.printf("Wald: (%g, %g)\n", ci#1, ci#2);
ci = waldAdjLU(c, y, n, alpha);
fo.printf("Adjusted Wald: (%g, %g)\n", ci#1, ci#2);
ci = wilsonLU(c, y, n, alpha);
fo.printf("Wilson Wald: (%g, %g)\n", ci#1, ci#2);

fo.printf("\n");

c = [0.5,0.5,-0.5,-0.5];
fo.writeline("c = [0.5,0.5,-0.5,-0.5]");

ci = waldLU(c, y, n, alpha);
fo.printf("Wald: (%g, %g)\n", ci#1, ci#2);
ci = waldAdjLU(c, y, n, alpha);
fo.printf("Adjusted Wald: (%g, %g)\n", ci#1, ci#2);
ci = wilsonLU(c, y, n, alpha);
fo.printf("Wilson Wald: (%g, %g)\n", ci#1, ci#2);

fo.printf("\n");
c = [0.5,-0.5,0.5,-0.5];
fo.writeline("c = [0.5, -0.5, 0.5,-0.5]");

ci = waldLU(c, y, n, alpha);
fo.printf("Wald: (%g, %g)\n", ci#1, ci#2);
ci = waldAdjLU(c, y, n, alpha);
fo.printf("Adjusted Wald: (%g, %g)\n", ci#1, ci#2);
ci = wilsonLU(c, y, n, alpha);
fo.printf("Wilson Wald: (%g, %g)\n", ci#1, ci#2);


// example 2


x=[115,121,131,141,151,161,176,190];
g=x.length;
c = (x - x.mean) / sum((x-x.mean).^2)

y = [3,17,12,16,12,8,16,8];
n = [156,252,284,271,139,85,99,43];


fo.printf("\n\n\n Example 2\n");
fo.printf("y = [3,17,12,16,12,8,16,8]\n n = [156,252,284,271,139,85,99,43]\nalpha=0.05\n");
fo.printf("c = [");
for k = 1 : g - 1
   fo.printf("%g,", c[k]);
end
fo.printf("%g)\n", c[g]);

fo.printf("\n");
ci = waldLU(c, y, n, alpha);
fo.printf("Wald: (%g, %g)\n", ci#1, ci#2);
ci = waldAdjLU(c, y, n, alpha);
fo.printf("Adjusted Wald: (%g, %g)\n", ci#1, ci#2);
ci = wilsonLU(c, y, n, alpha);
fo.printf("Wilson Wald: (%g, %g)\n", ci#1, ci#2);

fo.close();
