

/*

        solve ODE ivp
                   y'(t) = dydt(t, y),    y(t0) = y0
        using Runge-Kutta-Verner 5th order - 6th order method - 
                 
        calling sequence
        (t, y) = rkv56(dydt, tspan, y0)

		  tspan is a vector of 2 elements [t0, te]
		  y0 is initial value at t0

        public minimum and maxmimum step sizes, use
		  rkv56.hmin = h
		  rkv56.hmax = h
		  to rest the min and max step sizes

		  to check the number of func evals,
		  check rkv56.fevals after ivp is solved
*/


rkv56 = function [hmin = 1e-3, hmax = 10, tol = 1e-5, private fevals] (dydt, tv, y0) -> (t, y)

   n = y0.length;

   t0 = tv[1];
   te = tv[2];

   t[1] = tr = t0;
   y[1:n, 1] = yr = y0;

   this.fevals = 0;

   h = hmax;

   k = 2;

   while tr < te
        k1 = h * dydt(tr, yr);
	     k2 = h * dydt(tr + h/6, yr + k1/6);
	     k3 = h * dydt(tr + 4*h/15, yr + 4*k1/75 + 16*k2/75);
	     k4 = h * dydt(tr + 2*h/3, yr + 5*k1/6 - 8*k2/3 + 5*k3/2);
        k5 = h * dydt(tr + 5*h/6, yr - 165*k1/64 + 55*k2/6 - 425*k3/64 + 85*k4/96);
	     k6 = h * dydt(tr + h, yr + 12*k1/5 - 8*k2 + 4015*k3/612 - 11*k4/36 + 88*k5/255);
	     k7 = h * dydt(tr + h/15, yr - 8263*k1/15000 + 124*k2/75 - 643*k3/680 - 81*k4/250 + 2484*k5/10625);
	     k8 = h * dydt(tr + h, yr + 3501*k1/1720 - 300*k2/43 + 297275*k3/52632 - 319*k4/2322 + 24068*k5/84065 + 3850*k7/26703);
	     
	     R = max (abs(k1/160 + 125*k3/17952 - k4/144 + 12*k5/1955 + 3*k6/44 - 125*k7/11592 - 43*k8/616) / h);
	     q = 0.87 * (tol / R ) ^ (1/5);
	     this.fevals += 8; 
	     
	     if (R < tol)
           yr += 3*k1/40 + 875*k3/2244 + 23*k4/72 + 264*k5/1955 + 125*k7/11592 + 43*k8/616;
	        tr += h;
        
           t[k] = tr;
           y[1:n, k] = yr;
	        ++k;	
	     end;
	     
	     h = min(max (q, 0.1), 4.0) * h;
        
	     if (h > hmax)
             h = hmax;
        end;
        
	     if (tr + h > te)
             h = te - tr;
        elseif (h < hmin)
	        "Solution requires step size smaller than minimum";
             h = hmin;
	     end;
   end;

end;
